% Code for Example 4.5 in the manuscript.
%
% This example focuses on the running time of the
% structured algorithm w.r.t. matrix block size.
% (Block sizes are called m_i in the manuscript and 
% uniform block size is called bsize in this code).
%
% The test matrices are defined by random
% quasiseparable generators of order 1.
%
% This example is meant to show that the running
% times grow as bsize^3.
%

% Set up test parameters.

N=2; % size of A (in terms of blocks)
M=2;  % number of shifts
rL=1; % lower qs order
rU=1; % upper qs order

% Set block sizes. Large values are recommended in order
% to see the asymptotic behaviour.
minbsize=400;
stepbsize=400;
maxbsize=3200;
nbsize=(maxbsize-minbsize)/stepbsize + 1;

timings=zeros(1,nbsize);

% Begin loop over bsize
for bsize=minbsize:stepbsize:maxbsize  
    
    fprintf('%s %4d\n','Block size ',bsize)
    
    m=bsize*ones(N,1); % vector of block sizes
    time_sum=0;
    
    % perform the computation 10 times
    for trial_counter=1:10
        fprintf('%s %4d\n','Trial ',trial_counter)
        p=rand(bsize,rL,N); % N x rL (but first row not used)
        q=rand(rL,bsize,N-1);   % rL x (N-1)
        a=rand(rL,rL,N-1); % rL x rL x (N-1)  but first matrix not used
        g=rand(bsize,rU,N-1); % (N-1) x rU
        h=rand(rU,bsize,N);   % rU x N but first column not used
        b=rand(rU,rU,N-1); % rU x rU x (N-1)  but first matrix not used
        d=rand(bsize,bsize,N);
        y=rand(bsize,N);  % storing blocks horizontally
        sigma=1i*rand(M,1);  % shifts
        
        tic
        x=qs_shifted_function_blocks(p,q,a,g,h,b,d,sigma,y);
        time_sum=time_sum+toc;
    end
    indexbsize=(bsize-minbsize)/stepbsize + 1;
    timings(indexbsize)=time_sum/10;
end

% display results in a table
fprintf('%s %s\n','block size','time (sec)')
for j=1:nbsize
    fprintf('%4d %10.2e\n',minbsize+(j-1)*stepbsize,timings(j))
end

% show log-log plot
figure(1)
xdata=log10(minbsize:stepbsize:maxbsize);
ydata=log10(timings);
plot(xdata,ydata,'rd')
P = polyfit(xdata,ydata,1);
yfit = P(1)*xdata+P(2);
hold on;
plot(xdata,yfit,'k-');
legend('data','linear fit')
hold off;
title('Ex. 4.5')
xlabel('log_{10}(bsize)')
ylabel('log_{10}(time)')
fprintf('%s%10.2e\n','The slope for the linear fit is ',P(1))