function y=matxvec_blocks(p,q,a,g,h,b,d,x)
% Fast multiplication of a quasiseparable matrix times a vector: y = A x.
% Input: quasiseparable block generators for matrix A, block vector x 
% (blocks of x are arranged columnwise, so x is actually a matrix). 
% Output: block vector y.
%
% This function is for the block case. See matxvec if A is given 
% by a scalar quasiseparable representation.
%
% This is an implementation of Algorithm 13.1 from Eidelman, Gohberg,
% Haimovici, "Separable Type Representation of Matrices and Fast
% Algorithms", vol. 1. See also section 1.3.
%
% Warning: no check on size consistency.

[m,N]=size(x);  % m = block size, N = matrix size (in blocks)
yL=zeros(m,N);
yD=zeros(m,N);
yU=zeros(m,N);

yL(:,1)=zeros(m,1);
khi=q(:,:,1)*x(:,1);
yL(:,2)=p(:,:,2)*khi;
for i=3:N
    khi=a(:,:,i-1)*khi+q(:,:,i-1)*x(:,i-1);
    yL(:,i)=p(:,:,i)*khi;
end

for i=1:N
    yD(:,i)=d(:,:,i)*x(:,i);
end

yU(:,N)=zeros(m,1);
eta=h(:,:,N)*x(:,N);
yU(:,N-1)=g(:,:,N-1)*eta;

for i=N-2:-1:1
    eta=b(:,:,i+1)*eta+h(:,:,i+1)*x(:,i+1);
    yU(:,i)=g(:,:,i)*eta;
end

y=yL+yD+yU;

end